///////////////////////////////////////////////
// Jxam.class
//
// Main entry point for Jxam.
// Presents initial startup screens, reads the database of questions
// and selects a random subset.
// Then kicks off the main UI
//
// Mod Log:
// 7aug99  changed splash screen to a timed based display
// 14may99 Made Jxam into a separate thread.
///////////////////////////////////////////////

import java.util.Vector;
import java.util.HashSet;
import javax.swing.UIManager;
import javax.swing.JOptionPane;
import javax.swing.ImageIcon;

import java.io.File;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.FileOutputStream;




public class Jxam extends Thread{

    static final int MAX_NUM_QUESTIONS = 59;
    final public static String versionNumber = "v2.0";
    final public static String   progName = "Jxam (" + versionNumber + ")";
    public static void main(String [] args){

         Jxam Jx = new Jxam();
         Jx.start();
    }


    public void run(){

        //
        // Pop up initial splash screen
        // and populate with the LnF's
        UIManager myUIMgr = new UIManager();
        UIManager.LookAndFeelInfo[] installedLnFs = myUIMgr.getInstalledLookAndFeels();
        Object[] options = new Object[installedLnFs.length];

        for (int i = 0; i < installedLnFs.length; i++)
        {
           options[i] = installedLnFs[i].getName();
        }


        int option = JOptionPane.showOptionDialog(null, "Select a Look and Feel", progName,
                               JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
                               new ImageIcon("Splash.jpeg"), options, options[0]);

        // Set LnF to selected
        try
        {
            myUIMgr.setLookAndFeel(installedLnFs[option].getClassName());
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }



        // prompt for number of questions.
        String inputValue = null;
        int numQuestions = 0;


        // create a temporary questions database, to allow the total number
        // of questions in the database to be known.
        QuestionDatabase t = new QuestionDatabase(1);


        while( (inputValue == null) || (numQuestions == 0) || (numQuestions > MAX_NUM_QUESTIONS) ){
            try{
                inputValue = JOptionPane.showInputDialog("Enter number of questions \n(Max. = " + MAX_NUM_QUESTIONS + "   Available in Database = " + t.getTotalDatabaseSize() + ")" );
                numQuestions = Integer.parseInt(inputValue);
                }catch (NullPointerException ne){
                    inputValue = null;
                }
                catch (NumberFormatException nf){
                    inputValue = null;
                }
        }



        // free up temporary database object
        t = null;


        // create new database
        QuestionDatabase q = new QuestionDatabase(numQuestions);


        // create and start the GUI
        MainFrontEnd ui = new MainFrontEnd(q);
        ui.startUI();
    }



    public static String getVersion()
    {
        return versionNumber;
    }


    public static String getNameAndVersion()
    {
         return progName;
    }





}




