//////////////////////////////////////////////////
// MainfrontEnd.class
//
//
// This is the main UI class for the mock exam.
// It takes care of all the UI functions.
//
// Mod Log:
// 15apr00 - modified the way the online help is accessed.
// 16jun99 - fixed problem with stats time/question calc
// 30may99 - Modified Statistics to be a general summary
// 15may99 - Added in a label for questions
//         - modified to trap window close events.
//         - left justified the answers in the panel and removed
//           borders for readability
// 14may99 - moved marking of each question into Ques class.
// 9may99  - mod'ed to use the TextDisplay class
// 29apr99 - added in additional statistics info
//         - fixed so help text is available directly from JAR
// 14apr99 - Typo in report output
//
/////////////////////////////////////////////////

import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Font;
import java.awt.Dimension;
import java.awt.TextArea;
import java.awt.FlowLayout;
import java.awt.event.*;
import java.util.Vector;
import java.util.Date;
import javax.swing.border.BevelBorder;
import javax.swing.JComponent;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JRadioButton;
import javax.swing.AbstractButton;
import javax.swing.JOptionPane;
import javax.swing.ImageIcon;
import javax.swing.Timer;
import javax.swing.JFrame;
import javax.swing.BoxLayout;
import java.io.RandomAccessFile;
import java.io.ObjectInputStream;
import java.io.InputStream;
import java.io.EOFException;



public final class MainFrontEnd extends JFrame{

    // various containers
    JPanel         contentPane;
    JTextField     timerTextFld;
    JFrame         topLevelFrame;      // main frame
    JTextArea      QuestionTextArea;   // text area for question
    JScrollPane    qScrollPane;        // question scroll pane
    JScrollPane    aScrollPane;        // answer scroll pane
    JPanel         timerPanel;         // top panel (quit, mark, timer, etc)
    JPanel         AnswerPanel;        // panel for containing answers
    JSplitPane     QAPane;             // main split pane
    JLabel         quesLbl;            // question number label
    JRadioButton   tagForReview;       // review tag


    final int      INIT_QUESTION = 0;
    final String   ANSWER_STR = "Answer ";
    final String   NL = "\r\n";
    final int      FRAMEX = 850;
    final int      FRAMEY = 700;
    final int      Q_TEXT_X = 5;
    final int      Q_TEXT_Y = 5;
    final int      MAX_EXAM_TIME_SECONDS = 60 * 60 * 2;
    final int      ANS_TEXT_AREA_COLS = 30;
    final int      ANS_TEXT_AREA_ROWS = 4;


    int            initTimerVal = 0;
    int            currentQuestionNumber = INIT_QUESTION;
    int            NumberOfQuestions = 0;
    QuestionDatabase QDB;
    int            secondsCounter;
    int            warningCounterLimit;
    boolean        ended = false;




    public MainFrontEnd(){}

    public MainFrontEnd(QuestionDatabase q){

        // set local variables
        QDB = q;
        NumberOfQuestions = QDB.getNumQuestions();


        /////////////////////
        /// Set up top Level frame
        /////////////////////

        topLevelFrame = new JFrame(Jxam.getNameAndVersion());
        topLevelFrame.setSize(FRAMEX,FRAMEY);
        topLevelFrame.setResizable(true);
        topLevelFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        topLevelFrame.addWindowListener(new windowListener());


        // Set up content pane of the topLevelFrame
        contentPane = new JPanel(new BorderLayout());
        contentPane.setSize(FRAMEX, FRAMEY);
        contentPane.setBorder(new BevelBorder(BevelBorder.RAISED));
        topLevelFrame.getContentPane().add(contentPane);


        // set up navigation panel
        JPanel navContainer = new JPanel();

        JButton prevBtn = new JButton("Previous", new ImageIcon("prev.gif"));
        prevBtn.addActionListener(new prevBtnListener());
        prevBtn.setToolTipText("Previous Question");
        navContainer.add(prevBtn);

        try{
            JButton nextBtn = new JButton("Next", new ImageIcon("next.gif"));
            nextBtn.addActionListener(new nextBtnListener());
            nextBtn.setToolTipText("Next Question");
            nextBtn.setHorizontalTextPosition(nextBtn.LEADING);

            navContainer.add(nextBtn);
        }
        catch (IllegalArgumentException ia){}

        contentPane.add(navContainer, BorderLayout.SOUTH);



        //set up top container
        JPanel tsp = new JPanel();

        // set up timer panel
        timerPanel = new JPanel();
        timerPanel.setLayout(new BorderLayout());


        //set up question label
        JPanel quesPnl = new JPanel();
        quesLbl = new JLabel("Question #   ");
        quesLbl.setFont(new Font("Ariel", Font.BOLD, 16));
        quesLbl.setForeground(Color.black);
        quesPnl.add(quesLbl);

        timerPanel.add(quesPnl, BorderLayout.CENTER);


        //set up timer field
        timerTextFld = new JTextField(8);
        timerTextFld.setEditable(false);
        timerTextFld.setBackground(Color.white);
        timerTextFld.setFont(new Font("Arial", Font.ITALIC, 14));
        timerTextFld.setToolTipText("Time Remaining in exam");
        tsp.add(timerTextFld);

        // calc timer value based on number of questions
        float timePerQues = (MAX_EXAM_TIME_SECONDS)/59;
        secondsCounter = (int)(timePerQues * NumberOfQuestions);
        initTimerVal = secondsCounter;
        warningCounterLimit = (int)(secondsCounter/4);


        // Add HELP button
        JButton helpBtn = new JButton("Help");
        helpBtn.addActionListener(new helpBtnListener());
        helpBtn.setToolTipText("Exam Help");
        tsp.add(helpBtn);


        // Add Summary Button
        JButton summaryBtn = new JButton("Summary");
        summaryBtn.addActionListener(new summaryBtnListener());
        summaryBtn.setToolTipText("Exam Summary");
        tsp.add(summaryBtn);


        // set up timer panel
        timerPanel.add(tsp, BorderLayout.EAST);

        Timer t = new Timer(1000, new timerListener());
        t.start();
        contentPane.add(timerPanel, BorderLayout.NORTH);


        // add QUIT, MARK buttons
        JPanel opsContainer = new JPanel();

        JButton exitBtn = new JButton("Quit");
        exitBtn.addActionListener(new exitBtnListener());
        exitBtn.setToolTipText("Quit exam");

        JButton markBtn = new JButton("Mark");
        markBtn.addActionListener(new markBtnListener());
        markBtn.setToolTipText("Mark exam");

        opsContainer.add(exitBtn);
        opsContainer.add(markBtn);



        timerPanel.add(opsContainer, BorderLayout.WEST);

        // Set up the question/answer pane
        QAPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        QAPane.setContinuousLayout(false);
        QAPane.setDividerSize(5);

        // set up answer panel
        AnswerPanel = new JPanel();
        AnswerPanel.setLayout(new BorderLayout());
        aScrollPane = new JScrollPane(AnswerPanel);
        Dimension minimumSize = new Dimension( 400, 400 );
        aScrollPane.setMinimumSize(minimumSize);
        aScrollPane.setPreferredSize(minimumSize);
        QAPane.setBottomComponent(aScrollPane);

        // set up question panel
        QuestionTextArea = new JTextArea(Q_TEXT_X, Q_TEXT_Y);
        QuestionTextArea.setLineWrap(true);
        QuestionTextArea.setWrapStyleWord(true);
        QuestionTextArea.setOpaque(true);
        QuestionTextArea.setEditable(false);
        qScrollPane = new JScrollPane(QuestionTextArea);
        minimumSize = new Dimension( 300, 400 );
        qScrollPane.setMinimumSize(minimumSize);
        qScrollPane.setPreferredSize(minimumSize);


        qScrollPane.setViewportView(QuestionTextArea);
        qScrollPane.setViewportBorder(BorderFactory.createLineBorder(Color.yellow, 10));
        QAPane.setTopComponent(qScrollPane);

        // Add split pane to frame
        contentPane.add(QAPane, BorderLayout.CENTER);
        topLevelFrame.setContentPane(contentPane);
        topLevelFrame.setVisible(true);

    }


    private Question getCurrentQuestion(){
        return (Question)getCurrentQuestion(currentQuestionNumber);
    }

    private Question getCurrentQuestion(int questionNumber){
        return QDB.getQuestion(questionNumber);
    }


    private Vector getAnswers(Question question){
        return question.getPossibleAnswers();
    }



    public void startUI(){

        rePopulate(INIT_QUESTION);

    }




    // Re-populates the question and answer panes when
    // navigation buttons are pressed
    //
    private void rePopulate(int quesNum){
        String      s;
        int         temp = quesNum + 1;
        Question    question = getCurrentQuestion(quesNum);


        // Set up question
        QuestionTextArea.setText(null);
        QuestionTextArea.setToolTipText("Text of Question # " + temp);

        //update question label
        quesLbl.setText("Question # : " + temp);
        quesLbl.setToolTipText("Question " + temp + " of " + NumberOfQuestions);


        if (question.MultipleSelection() == true){
            s = "Select ALL valid answers...";
        }
        else{
            s = "Select the 1 correct answer...";
        }

        QuestionTextArea.append(s + "\n\n\n");

        QuestionTextArea.setBackground(Color.yellow);
        QuestionTextArea.append(question.getQuestionText());

        // Clear the answer panel
        AnswerPanel.removeAll();
        topLevelFrame.invalidate();


        if (question.MultipleSelection() == true){
            populateMultipleAnswers(quesNum);
        }
        else{
            populateSingleAnswers(quesNum);
        }

        QuestionTextArea.setCaretPosition(0);

        aScrollPane.revalidate();
        qScrollPane.revalidate();

    }


    // Re-populates a multiple selection answer pane
    private void populateMultipleAnswers(int quesNum){

        String     ansText;
        Answer     ans;
        int        x = 0;
        JCheckBox  ch;
        JPanel     tempPanel;
        boolean    state = false;


        Question question = QDB.getQuestion(quesNum);

        Vector possAns = question.getPossibleAnswers();

        AnswerPanel.setLayout(new BoxLayout(AnswerPanel, BoxLayout.Y_AXIS));

        for (int i = 0; i < possAns.size(); i++){

            // get the answer
            ans =(Answer)(possAns.elementAt(i));
            ansText = ans.getPossibleAnswerText();

            state = ans.isSelected();

            JPanel singleAnsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));


            // temp panel to hold checkbox and answer
            tempPanel = new JPanel();

            x = i + 1;

            ch = new JCheckBox(ANSWER_STR + x, state);
            ch.addItemListener(new CheckboxListener());
            tempPanel.add(ch);

            JTextArea aText = new JTextArea(ansText, ANS_TEXT_AREA_ROWS, ANS_TEXT_AREA_COLS);
            aText.setLineWrap(true);
            aText.setEditable(false);
            aText.setOpaque(true);
            aText.setToolTipText("Possible Answer #" + x);
            aText.setWrapStyleWord(true);
            aText.setCaretPosition(0);

            JScrollPane qSP = new JScrollPane(aText);

            tempPanel.add(qSP);


            singleAnsPanel.add(tempPanel);

            AnswerPanel.add(singleAnsPanel);


        }

    }



    // Re-populates a single answer selection pane
    private void populateSingleAnswers(int quesNum){

        Answer                      ans;
        ButtonGroup buttonGroup =   new ButtonGroup();
        String                      ansText;
        int                         x;
        boolean                     state;
        JRadioButton                radioButton;
        JPanel                      tempPanel;

        Question question = QDB.getQuestion(quesNum);
        Vector possAns = question.getPossibleAnswers();


        AnswerPanel.setLayout(new BoxLayout(AnswerPanel, BoxLayout.Y_AXIS));

        for (int i = 0; i < possAns.size(); i++){

            // get the answer
            ans =(Answer)(possAns.elementAt(i));
            ansText = ans.getPossibleAnswerText();

            state = ans.isSelected();

            JPanel singleAnsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));


            // temp panel to hold checkbox and answer
            tempPanel = new JPanel();

            x = i + 1;


            radioButton = new JRadioButton(ANSWER_STR + x);
            radioButton.setSelected(state);
            radioButton.addItemListener(new CheckboxListener());
            tempPanel.add(radioButton);

            buttonGroup.add(radioButton);

            JTextArea aText = new JTextArea(ansText, ANS_TEXT_AREA_ROWS, ANS_TEXT_AREA_COLS);
            aText.setLineWrap(true);
            aText.setOpaque(true);
            aText.setEditable(false);
            aText.setToolTipText("Possible Answer #" + x);
            aText.setWrapStyleWord(true);
            aText.setCaretPosition(0);

            JScrollPane qSP = new JScrollPane(aText);

            tempPanel.add(qSP);

            singleAnsPanel.add(tempPanel);

            AnswerPanel.add(singleAnsPanel);

        }

    }




    // intercepts events from the checkboxes and radio buttons
    private class CheckboxListener implements ItemListener{
        public void itemStateChanged(ItemEvent ie){

            Question currQues = getCurrentQuestion();
            Vector ansVector = getAnswers(currQues);
            AbstractButton selectedAns;

            selectedAns = (AbstractButton)ie.getItem();
            Answer ans;
            int tempInt;
            String tempStr = ANSWER_STR;
            String btnText = selectedAns.getText();


            // find and mark the selected answer
            for (int i = 0; i < ansVector.size(); i++){
                tempInt = i + 1;
                if (btnText.equals(tempStr + tempInt)){
                    tempInt = ie.getStateChange();
                    ans = (Answer)ansVector.elementAt(i);

                    if (tempInt == ie.SELECTED){
                        if (!currQues.MultipleSelection()){
                            currQues.deSelectAllAns();
                        }

                        ans.setSelectedAns();
                    }
                    if (tempInt == ie.DESELECTED){
                        if (currQues.MultipleSelection()){
                            ans.clearSelectedAns();
                        }

                    }
                }
            }


        }
    }



    // intercepts the "Previous" button presses
    private class prevBtnListener implements ActionListener{

       public void actionPerformed(ActionEvent ae){
           if (currentQuestionNumber == INIT_QUESTION){
               currentQuestionNumber = NumberOfQuestions - 1;
           }
           else{
               currentQuestionNumber--;
           }

           rePopulate(currentQuestionNumber);
        }
    }



    // intercepts the "Previous" button presses
    private class nextBtnListener implements ActionListener{

       public void actionPerformed(ActionEvent ae){

           if (currentQuestionNumber == NumberOfQuestions - 1){
               // rotate/wrap around
               currentQuestionNumber = INIT_QUESTION;
           }
           else{
               currentQuestionNumber++;
           }

           rePopulate(currentQuestionNumber);
       }
   }


    // intercepts "Exit" button presses
    private class exitBtnListener implements ActionListener{
       public void actionPerformed(ActionEvent ae){
           OK_Cancel();
       }
   }



    // intercepts "Summary" button presses
    private class summaryBtnListener implements ActionListener{
       public void actionPerformed(ActionEvent ae){

           String [] headings = {"Question", "Attempted", "Correct"};

           int x = headings.length;
           Object [][] data = new Object[QDB.getNumQuestions()][x];

           for (int i = 0; i < QDB.getNumQuestions(); i++){

               Question question = QDB.getQuestion(i);

               data[i][0] = new Integer(i + 1);  // question
               data[i][1] = new Boolean(question.isAttempted());  // attempted y/n
               data[i][2] = new Boolean(question.isCorrect());
               //data[i][3] = new TextArea(question.getQuestionText());

           }

           String statsText = "Summary " + "\n\n" + calcStats();
           TableDisplay t = new TableDisplay(topLevelFrame, "Exam Summary", headings, data, statsText);

       }
    }




    private String calcStats(){


       Question question;
       int correctAnswers = 0;

       // get num of questions answered.
       int questionsAnswered = 0;
       for (int i = 0; i < NumberOfQuestions; i++){
           question = QDB.getQuestion(i);
           if (question.isAttempted()){
               questionsAnswered++;
           }
       }
       String strQuesAns = "Questions Answered : " + questionsAnswered + "\n";



       // Number of questions remaining

       int quesRemaining = NumberOfQuestions - questionsAnswered;
       String strQuesRemaining = "Questions Remaining : " + quesRemaining + "\n";



       // Get avg time per question
       String strAvgQTime;
       if (questionsAnswered != 0){
           int avgTimePerQ = (initTimerVal - secondsCounter) / questionsAnswered;

           strAvgQTime = "Avg. Time per Question = " + avgTimePerQ + "secs\n";
           //System.out.println("Avg Time = " + avgTimePerQ);
       }
       else{
           strAvgQTime = "Avg. Time per Question = ----\n";
       }


       // Get number of correct answers so far
       String strCorrAns;
       for (int i = 0; (i < NumberOfQuestions); i++){

           // get question
           question = QDB.getQuestion(i);

           // Check for correct answers
           if (question.markQuestion() == true){
               correctAnswers++;
           }
       }
       strCorrAns = "Correct Answers so far : " + correctAnswers + "\n";


       // Calc percentage correct
       String strPCCorrect = "% (based on # questions answered) : ";
       int percent;

       if (questionsAnswered != 0)
           percent = (correctAnswers * 100)/questionsAnswered;
       else
           percent = 0;

       strPCCorrect += percent + "%\n";



       return (strQuesAns + strQuesRemaining  + strCorrAns + strPCCorrect + strAvgQTime);

   }


    // intercepts "Mark" button presses
   private class markBtnListener implements ActionListener{
       public void actionPerformed(ActionEvent ae){

           int option = 0;


           // do a check to see if all questions are attempted

           if (allAnswered() == false){
               // all questions not attempted, pop-up dialog
               Object[] options = { "MARK", "CANCEL" };

               option = JOptionPane.showOptionDialog(null, "Press MARK to mark exam \nPress CANCEL to continue test", "All questions not attempted!",
                           JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
                           null, options, options[0]);

               if (option != JOptionPane.OK_OPTION){
                   // continue exam
                   return;
               }
           }


           // Mark all/any answers and exit
           markAnswers();

       }
   }






   // intercepts "Help" button presses
   private class helpBtnListener implements ActionListener{
       public void actionPerformed(ActionEvent ae){

           RandomAccessFile raf;
           String line = " ";
           String helpText = "";
           TextDisplay helpTxtArea = new TextDisplay(topLevelFrame, "Jxam Help", true);

           try{
               raf = new RandomAccessFile("jxam help.txt", "r");

               while ( (line = raf.readLine()) != null)
               {
                   helpText += line + "\n\r";
                }

                helpTxtArea.appendText(helpText);

           }catch(EOFException oefe){
               System.out.println("EOF");
           }catch(Exception e){
               e.printStackTrace();
               System.out.println("Exc");
           }
           // display the help window.
           helpTxtArea.display();

       }
   }

   private boolean allAnswered(){

       boolean quesAttempted = false;
       boolean AllAnswered = true;
       Vector ansVector;
       Answer answer;
       Question question;

       questLoop:
           for (int i = 0; (i < NumberOfQuestions) && (AllAnswered == true); i++){
               question = QDB.getQuestion(i);
               ansVector = question.getPossibleAnswers();

               for (int j = 0; j < ansVector.size(); j++){
                   answer = (Answer)ansVector.elementAt(j);

                   if (answer.isSelected()){
                       quesAttempted = true;
                       continue questLoop;
                   }
               }

               AllAnswered = false;
           }

       return AllAnswered;
   }


   private void markAnswers()
	{
		topLevelFrame.setVisible (false);

		java.util.List qs = new java.util.Vector();
		for (int i = 0; i < QDB.getNumQuestions(); i++) qs.add (QDB.getQuestion (i));

		ExamSummary es = new ExamSummary (qs);
		es.addWindowListener (new WindowAdapter()
			{
				public void windowClosing (WindowEvent e)
				{
					MainFrontEnd.this.topLevelFrame.setVisible (true);
				}
			});

		Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		es.setSize (screenSize.width, screenSize.height);
		es.setVisible (true);

   }



   class timerListener implements ActionListener{
       public void actionPerformed(ActionEvent ae){

           int hrs = (int)(secondsCounter/3600);
           int min = (int)(secondsCounter/60 - hrs*60);
           int sec = (int)(secondsCounter - hrs*3600 - min*60);
           --secondsCounter;

           if (ended == true)
               return;

           if (secondsCounter <= 0){
               ended = true;

               // timer expired.
               Object[] options = { "OK"};

               int option = JOptionPane.showOptionDialog(null, "Press OK to mark exam", "Time Limit Expired!!!",
                           JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
                           null, options, options[0]);

               markAnswers();

           }


           // change colour based on time remaining
           if (secondsCounter <= warningCounterLimit){
               timerTextFld.setBackground(Color.red);
               timerTextFld.setForeground(Color.white);
           }


           // update timer value
           timerTextFld.setText(hrs + "h:" + min + "m:" + sec + "s");
       }

   }


   private class windowListener extends WindowAdapter{

       public void windowClosing(WindowEvent we){
           OK_Cancel();
       }
   }


   // pop-up an exit dialog
   private void OK_Cancel(){

       Object[] options = { "OK", "CANCEL" };

       int option = JOptionPane.showOptionDialog(null, "Press OK to exit exam \nPress CANCEL to continue test", "Are you sure?",
                          JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
                          null, options, options[0]);

       if (option == JOptionPane.OK_OPTION)
           System.exit(0);

       return;
   }



}
