import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Panel with navigation buttons that allow to move from one question
 * to another.
 *
 * @author	JPL
 */
public class NavButtonsBar extends JPanel implements ActionListener
{
	private NavButtonListener listener;

	/**
	 * Constructs a new <code>NavButtonsBar</code>.
	 *
	 * @param		listener	         object interested in getting button press events
	 * @param		enabled           whether "next/prev" buttons are enabled
	 * @param		correctEnabled    whether "next/prev correct" are enabled
	 * @param		incorrectEnabled  whether "next/prev incorrect" are enabled
	 */
	public NavButtonsBar (
		NavButtonListener listener,
		boolean enabled, boolean correctEnabled, boolean incorrectEnabled)
	{
		this.listener = listener;

		setLayout (new BoxLayout (this, BoxLayout.X_AXIS));

		JButton firstBtn, lastBtn;

		add (firstBtn = createButton (
			"Save", "Saves the exam summary to disk",
			"SAVE", true));
		add (Box.createHorizontalGlue());
		add (createButton (
			"Prev Correct", "Displays previous correctly answered question",
			"PREV_CORRECT", correctEnabled));
		add (createButton (
			"Next Correct", "Displays next correctly answered question",
			"NEXT_CORRECT", correctEnabled));
		add (Box.createHorizontalStrut (15));
		add (createButton (
			"Prev Incorrect", "Displays previous incorrectly answered question",
			"PREV_INCORRECT", incorrectEnabled));
		add (createButton (
			"Next Incorrect", "Displays next incorrectly answered question",
			"NEXT_INCORRECT", incorrectEnabled));
		add (Box.createHorizontalStrut (15));
		add (createButton (
			"Prev", "Displays previous question", 
			"PREV", enabled));
		add (lastBtn = createButton (
			"Next", "Displays next question",
			"NEXT", enabled));

		lastBtn.setNextFocusableComponent (firstBtn);

		setBorder (BorderFactory.createCompoundBorder (
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder (2, 2, 2, 2)));
	}

	public void actionPerformed (ActionEvent e)
	{
		listener.buttonClicked (
			((AbstractButton) e.getSource()).getActionCommand() );
	}

	public Dimension getMaximumSize ()
	{
		return new Dimension (
			super.getMaximumSize().width, getPreferredSize().height);
	}

	private JButton createButton (
		String btnLabel, String tip, String actionCommand, boolean enabled)
	{
		JButton btn = new JButton (btnLabel);
		btn.addActionListener (this);
		btn.setActionCommand (actionCommand);
		btn.setAlignmentY (Component.TOP_ALIGNMENT);
		btn.setEnabled (enabled);
		btn.setToolTipText (tip);
		return btn;
	}
}