/////////////////////////////////////////
// Question.class
//
// Contains a class which represents a question
// in the exam.
//
// Mod Log:
// 04aug99 Added in a question reference number
// 30may99 Added field "for review" for later use
// 08may99 Added in explanation
////////////////////////////////////////

import java.util.Vector;
import java.io.Serializable;


class Question implements Serializable{

     transient public final static int MAX_ANSWERS_PER_QUESTION = 5;

     private String QuestionText;
     private String Explanation;
     transient private boolean forReview = false;
     private Vector Answers;
     private boolean MultipleSelection;
     private final String   NL = "\r\n";

    public Question(){
         Answers = new Vector(MAX_ANSWERS_PER_QUESTION);
     }




     ////////////////////
     // Accessor Methods
     ////////////////////

     public String getQuestionText(){
        return QuestionText;
     }


     public static int getNumAnswersPerQuestion(){
         return MAX_ANSWERS_PER_QUESTION;
     }


     public boolean isForReview(){
         return forReview;
     }

     public Vector getPossibleAnswers(){
        return Answers;
     }

     public String getExplanation(){
         return Explanation;
     }


     public boolean MultipleSelection(){
         return MultipleSelection;
     }

     public int getNumCorrAns(){
         Vector v = this.getCorrectAnswers();

         return v.size();
     }

     public int getNumSelAns(){
         Vector v = this.getSelectedAnswers();

         return v.size();
     }



     public Vector getCorrectAnswers(){
         Vector correctAnswers = new Vector();

         for (int i = 0; i < Answers.size(); i++){

             Answer ans = (Answer)Answers.elementAt(i);

             if (ans.isCorrect()){
                 correctAnswers.addElement(ans);
             }
         }

         return correctAnswers;
     }


     public boolean isCorrect(){
         return markQuestion();
     }




     public Vector getSelectedAnswers(){

         Vector selectedAnswers = new Vector();

         for (int i = 0; i < Answers.size(); i++){

             Answer ans = (Answer)Answers.elementAt(i);

             if (ans.isSelected()){
                 selectedAnswers.addElement(ans);
             }
         }

         return selectedAnswers;
     }




     public boolean isAttempted(){

         Answer ans;
         boolean questionAttempted = false;

         for (int j = 0; j < Answers.size(); j++){

             ans = (Answer)Answers.elementAt(j);

             if (ans.isSelected()){
                 return(true);
             }
         }

         // if we get here, then the question hasn't been attempted
         return (false);
     }


     public String toString(){
         return this.toString(0);
     }




     public String toString(int questionNumber){
         int temp;
         int i;

         StringBuffer sbQ = new StringBuffer();

         questionNumber++;

         sbQ.append(NL + NL + NL + NL);
         sbQ.append("****************************************************");
         sbQ.append(NL + NL +"Question #" + questionNumber);

         if (this.isCorrect()){
             sbQ.append("  ****** CORRECT *****");
         }
         else{
             sbQ.append("  Incorrect");
         }


         sbQ.append(NL + "============" + NL + NL);
         sbQ.append(this.getQuestionText());
         sbQ.append(NL + NL);

         // Output possible answers
         Vector possibleAnswers = this.getPossibleAnswers();
         Answer ans;


         sbQ.append("Possible Answers:" + NL);
         sbQ.append("-------------------" + NL + NL);

         for (i = 0; i < possibleAnswers.size(); i++){
             temp = i + 1;
             sbQ.append("Possible Answer " + temp + ":" + NL);
             ans = (Answer)possibleAnswers.elementAt(i);
             sbQ.append(ans.getPossibleAnswerText() + NL + NL);
         }




         //
         // output correct answer(s)
         //
         String s;
         if (this.getNumCorrAns() == 1)
             s = "Correct Answer...";
         else
             s = "Correct Answers...";

         sbQ.append(s + NL);
         sbQ.append("-----------------" + NL);

         for (i = 0; i < possibleAnswers.size(); i++){

             ans = (Answer)possibleAnswers.elementAt(i);

             if (ans.isCorrect()){
                 temp = i + 1;
                 sbQ.append("Answer " + temp + ":" + NL);
                 sbQ.append(ans.getPossibleAnswerText() + NL);
             }
         }

         sbQ.append(NL);

         //
         // Output selected answer(s)
         //
         if (this.getNumSelAns() == 1)
             s = "Selected Answer...";
         else
             s = "Selected Answers...";

         sbQ.append(s + NL);
         sbQ.append("-------------------" + NL);

         if (this.getNumSelAns() == 0){
             sbQ.append("[none]" + NL);
         }
         else{
             for (i = 0; i < possibleAnswers.size(); i++){

                 ans = (Answer)possibleAnswers.elementAt(i);

                 if (ans.isSelected()){

                     temp = i + 1;
                     sbQ.append("Answer " + temp + ":" + NL);
                     sbQ.append(ans.getPossibleAnswerText() + NL);
                 }
             }
         }

         //
         // Output explanation
         //
         sbQ.append(NL + NL);
         sbQ.append("Explanation" + NL);
         sbQ.append("-------------------" + NL);
         sbQ.append(this.getExplanation() + NL);



         return sbQ.toString();
     }





    ///////////////////
    // Mutator Methods
    ///////////////////

    public void deSelectAllAns(){

        Answer ans;

        for (int i = 0; i < Answers.size(); i++){
            ans = (Answer)Answers.elementAt(i);
            ans.clearSelectedAns();
        }
    }

    public void setQuestionText(String text){
        QuestionText = text;
    }

    public void setExplanation(String s){
        Explanation = s;
    }


    public void addPossibleAnswer(Answer answer){
        Answers.addElement(answer);
    }

    public void setMulSel(boolean fl){
        MultipleSelection = fl;
    }


    public void setForReview(boolean review){
        forReview = review;
    }


    public boolean markQuestion(){

        Answer ans;

        // 2 pass - check all correct answers are selected
        for (int j = 0; j < Answers.size(); j++){
            ans =(Answer)(Answers.elementAt(j));

            if ( (ans.isCorrect() == true) && !(ans.isSelected() == true) ){
                // wrong answer,
                return false;
            }

        }

        // check all selected answers are correct
        for (int j = 0; j < Answers.size(); j++){

            ans =(Answer)(Answers.elementAt(j));

            if ( (ans.isSelected() == true) && !(ans.isCorrect() == true) ){
                // wrong answer,
                return false;
            }

        }

        // if we get here the question is correct
        return true;
     }

}