////////////////////////////////////////////////
// QuestionDatabase.class
//
// Class to interface to the question database (on disk)
// and maintain runtime database of questions
//
//
// Mod Log:
// ========
//
// 04aug99 Added in a question reference
// 8may99  Added in explanation text
// 24apr99 changed to use InputStream rather than
//         FileInputStream (mod courtesy of Artur).
//
////////////////////////////////////////////////


import java.util.Vector;
import java.util.StringTokenizer;
import java.io.File;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.FileOutputStream;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.DriverManager;


// Class to access Q&A database

public class QuestionDatabase{

    File DBFile;
    Connection DBConn;
    Vector QuestionsDB;
    Vector tempQDB;
    int TotalQuestionsInDatabase;
    final String serialisedDBFile = "questions.db";



    // Constructor
    // Read in the serialised questions from the file, and
    // create a Vector of questions.
    public QuestionDatabase(int numQuestions){

        try{

            InputStream istream = QuestionDatabase.class.getResourceAsStream("questions.db");
            ObjectInputStream ip = new ObjectInputStream(istream);

            tempQDB = (Vector)ip.readObject();

            TotalQuestionsInDatabase = tempQDB.size();

            //create a random subset of questions from the entire database
            QuestionsDB = new Vector(numQuestions);

            Object obj;
            for (int i = 0; i < numQuestions; i++){
                //select a random element from tempQDB and add to QuestionsDB
                int x = (int)(Math.random() * tempQDB.size());

                obj = tempQDB.elementAt(x);
                QuestionsDB.addElement(obj);

                // remove from tempQDB
                tempQDB.removeElementAt(x);
             }

            //free up objects held by tempQDB
            tempQDB.clear();


        }catch(Exception e){
            e.printStackTrace();
        }

    }




    // Constructor
    // Read in from the database and serialise into a file
    // *** NOT USED ***
    // *** RETAINED FOR REFERENCE ONLY ***
    public QuestionDatabase(){

        Statement         stmt; // SQL statement object
        String            query;   // SQL select string
        ResultSet         rs;   // SQL query results
        boolean           more = false;   // "more rows found" switch
        String            v1, v2, v3; // Temporary storage results
        Vector            ExamList = new Vector(20,5);
        Question          ques;
        StringTokenizer   st;
        Answer            ans;

           // ODBC data source name
        String dsn = "jdbc:odbc:javaqa";


        try{
            Class.forName ("sun.jdbc.odbc.JdbcOdbcDriver");
            }catch( ClassNotFoundException ee) {
                ee.printStackTrace();
            }



         // Connect to the database
         try{
             DBConn = DriverManager.getConnection(dsn);

             // Shut off auto-commit
             //DBConn.setAutoCommit(false);
             }catch(SQLException se){
                 System.out.println("SQLException" + se);
             }


         query = "SELECT * FROM QuestionsAnswers";

         try{
             stmt = DBConn.createStatement();

             rs = stmt.executeQuery(query);

             // Check to see if any rows were read
             more = rs.next();
             if (!more) {
                 System.out.println("No rows found.");
                 return;
             }


              // Loop through the rows retrieved from the query
         int loop = 0;
         loop:
            while (more) {

                loop++;
                System.out.println("Question " + loop);
                boolean [] stAns = new boolean[10];
                String stNum;

                // set up question text
                ques = new Question();
                ques.setQuestionText(rs.getString("Question"));

                // get correct answers
                ans = new Answer();
                v2 = rs.getString("Answers");
                st = new StringTokenizer(v2, ",");
                while(st.hasMoreTokens()){
                    stNum = st.nextToken();
                    stNum.trim();
                    stAns[Integer.parseInt(stNum)] = true;
                }


                if (rs.getBoolean("Multiple Selection")){
                    ques.setMulSel(true);
                }
                else{
                    ques.setMulSel(false);
                }

                String ExplanationText = rs.getString("Explanation");
                ques.setExplanation(ExplanationText);


                for (int i = 1; i <= Question.getNumAnswersPerQuestion(); i++){
                    ans = new Answer();
                    v2 = "Poss Ans " + i;
                    v1 = rs.getString(v2);
                    if (v1 != null){
                        ans.PossibleAnswerText = v1;
                        if (stAns[i] == true)
                            ans.CorrectFlag = true;
                        else
                            ans.CorrectFlag = false;

                    // add answer to question
                    ques.addPossibleAnswer(ans);
                    }
                }



                // Add question to exam list
                ExamList.addElement(ques);


                // Check to see if any rows were read
                more = rs.next();
                if (!more) {
                    break loop;
                }

            }

            rs.close();
            stmt.close();


        }catch (SQLException se){
            System.out.println("SQLException " + se);
        }


        // now serialize the retrieved questions into a file

        File file = new File(serialisedDBFile);

        if (file.exists())
            file.delete();

        try{

            file.createNewFile();


            FileOutputStream ostream = new FileOutputStream(file);
            ObjectOutputStream p = new ObjectOutputStream(ostream);


            p.writeObject(ExamList);

            p.flush();
            ostream.close();
        }catch(Exception e){}


    }



    ///////////////////////////////////////////////
    // Accessor Methods
    //////////////////////////////////////////////
    public Question getQuestion(int i){
        return (Question)QuestionsDB.elementAt(i);
    }

    public int getNumQuestions(){
        return QuestionsDB.size();
    }

    public int getTotalDatabaseSize(){
        return TotalQuestionsInDatabase;
    }



}







