import javax.swing.*;
import java.awt.*;

/**
 * Panel containing question text, possible answers with indication which
 * ones were selected and if they are correct and an explanation for the question.
 *
 * @author	JPL
 */
public class QuestionDisplay extends JPanel
{
	private Question question;
	private TextScrollPane qText;
	private TextScrollPane explanation;
	private JPanel answersPanel;

	public QuestionDisplay ()
	{
		qText = new TextScrollPane ("", 15, 30);
		qText.setToolTipText ("Question text");
		qText.setAlignmentY (Component.TOP_ALIGNMENT);

		answersPanel = new JPanel();
		answersPanel.setLayout (new BoxLayout (answersPanel, BoxLayout.Y_AXIS));
		answersPanel.setAlignmentY (Component.TOP_ALIGNMENT);

		JPanel upper = new JPanel();
		upper.setAlignmentX (Component.LEFT_ALIGNMENT);
		upper.setLayout (new BoxLayout (upper, BoxLayout.X_AXIS));
		upper.add (qText);
		upper.add (Box.createHorizontalStrut (10));
		upper.add (answersPanel);

		explanation = new TextScrollPane ("", 5, 40);
		explanation.setAlignmentX (Component.LEFT_ALIGNMENT);
		explanation.setToolTipText ("Explanation of correct answers");

		setLayout (new BoxLayout (this, BoxLayout.Y_AXIS));
		add (upper);
		add (explanation);

		setBorder (BorderFactory.createEmptyBorder (5, 5, 5, 5));
	}

	/**
	 * Makes this panel display the given question (and its answers etc).
	 */
	public void setQuestion (Question question)
	{
		this.question = question;
		qText.setText (question.getQuestionText());
		explanation.setText (question.getExplanation());
		updateAnswersPanel();
	}

	private void updateAnswersPanel ()
	{
		answersPanel.removeAll();

		java.util.List answers = question.getPossibleAnswers();
		for (java.util.Iterator i = answers.iterator(); i.hasNext();)
		{
			Answer a = (Answer) i.next();
			answersPanel.add (new AnswerBox (a));
		}

		answersPanel.revalidate();
	}

	private static class AnswerBox extends JPanel
	{
		private Answer answer;

		public AnswerBox (Answer answer)
		{
			this.answer = answer;
			setAlignmentX (Component.LEFT_ALIGNMENT);
			setLayout (new BoxLayout (this, BoxLayout.X_AXIS));
			add (Box.createHorizontalStrut (5));
			add (createCheckBox (answer.isSelected(), "S:"));
			add (createCheckBox (answer.isCorrect(), "C:"));
			add (createAnswerText (answer.getPossibleAnswerText()));
		}

		private JCheckBox createCheckBox (boolean state, String label)
		{
			JCheckBox cb = new JCheckBox (label);
			cb.setHorizontalTextPosition (JCheckBox.LEFT);
			cb.setSelected (state);
			cb.setEnabled (false);
			cb.setAlignmentY (Component.TOP_ALIGNMENT);
			return cb;
		}

		private TextScrollPane createAnswerText (String text)
		{
			TextScrollPane tsp = new TextScrollPane (text, 3, 25);
			tsp.setAlignmentY (Component.TOP_ALIGNMENT);
			tsp.setToolTipText (
				(answer.isCorrect() ? "Correct" : "Wrong") + " answer");
			return tsp;
		}
	}
}